import os,sys
import requests
from numpy import pi, sqrt, arccos, cos, arctan2, float32, isnan
from datetime import datetime

# inspired from https://github.com/Bill-Gray/lunar/blob/master/mpc_code.cpp

EARTH_MAJOR_AXIS_IN_METERS = 6378137.
EARTH_MINOR_AXIS_IN_METERS = 6356752.314140347
B = EARTH_MINOR_AXIS_IN_METERS / EARTH_MAJOR_AXIS_IN_METERS
query = 'https://minorplanetcenter.net/iau/lists/ObsCodes.html'
outfile = 'MPCobs.csv'

def parralax_to_lat(c, s):
	fy = abs(s)
	fx = abs(c)
	if c == 0.:
		lat = pi / 2.
	else :
		c_squared = 1. - B**2
		e = (B * fy - c_squared) / fx
		f = (B * fy + c_squared) / fx
		p = (4. / 3.) * (e * f + 1.)
		q = 2. * (e * e - f * f)
		d = p * p * p + q * q
		if d >= 0.:
			sqrt_d = sqrt( d)
			v = (sqrt_d - q)**(1/3.) - (sqrt_d + q)**(1/3.)
		else:
			sqp = sqrt(-p)
			temp_ang = arccos(q / (sqp * p))
			v = 2. * sqp * cos(temp_ang / 3.)

		g = (sqrt( e * e + v) + e) * .5
		t = sqrt(g * g + (f - v * g) / (2. * g - e)) - g
		lat = arctan2((1. - t * t), 2. * B * t)

	if c < 0.:
		lat = pi - lat
	if s < 0.:
		lat = -lat
	lat *= 180. / pi
	return lat

def Run(savepath = None):

	out = ['#Date fetched:\t{:s}'.format(datetime.now().strftime("%d/%m/%Y"))]
	req = requests.request('GET', query)
	lines = [l for l in req.text.split('\n') if len(l) > 0]
	out += ['#Code\tLon\tLat\tDesc']
	for l in lines[2:]:
		#005   2.231000.659891+0.748875Meudon
		if len(l) < 30:
			continue
		code = l[0:3]
		desc = l[30:]
		try:
			lon = float32(l[3:13])
			c = float32(l[13:21])
			s = float32(l[21:30])
		except:
			continue
		if (c == 0. and s == 0.):
			continue
		lat = parralax_to_lat(c, s)
		if isnan(lat):
			continue
		out += ['{:s}\t{:8.6f}\t{:+9.6f}\t{:s}'.format(code, lon, lat, desc)]

	if len(out) > 1:
		if savepath is None:
			outpath = outfile
		else:
			outpath = os.path.join(savepath, outfile)
		with open(outpath, 'w') as f:
			f.write("\n".join(str(o) for o in out))
				


if __name__ == "__main__":

	args=[]
	kwargs={}
	for a in sys.argv[1:]:
		if '=' in a:
			f,v=a.split('=')
			kwargs[f]=v
		else:
			args.append(a)
	Run(*tuple(args),**kwargs)
	# c = 0.62411
	# s = 0.77873
	# lat = parralax_to_lat(c, s)
	# print('Lat:{:g}'.format(lat))